# -*- coding: utf-8 -*-
import logging
from twisted.python import log

def dbg(*a, **kw):
  kw["logLevel"] = logging.DEBUG
  log.msg(*a, **kw)

class CameraProtocol():
  def setSize(self, size):
    raise NotImplementedError()

  def getSize(self):
    raise NotImplementedError()

  def disconnect(self):
    dbg("CameraProtocol.disconnect not implemented")

  def updateSettings(self):
    dbg("CameraProtocol.updateSettings not implemented")

class Listener():
  def lostConnection(self, reason, failed, *a, **kw):
    raise NotImplementedError()

  def gotFrame(self, frame, *a, **kw):
    raise NotImplementedError()

  def forcedDisconnect(self, *a, **kw):
    dbg("Listener.forcedDisconnect not implemented")

class UnknownDevice(Exception):

  def __init__(self, address):
    self.address = address

  def __str__(self):
    return "Device unknown %s" % self.address



# -*- coding: utf-8 -*-
import sys

if sys.platform.startswith("linux"):
    from bluez import *
elif sys.platform.startswith("win"):
    from win import *
elif sys.platform.startswith("darwin"):
    from darwin.sock import *
else:
    raise Exception, "Platform not supported"




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time

from twisted.web import http, server
from twisted.internet import protocol
from twisted.internet import reactor
from twisted.internet import defer
from twisted.internet import task
from twisted.web.resource import Resource
from twisted.python import log
from logging import DEBUG
from airi.camera.protocol import CameraFactory
from airi.camera import Listener
from airi import report, RESULT
from functools import partial

MULTIPARTRESPONSE= "--%s\r\nContent-Type: %s\r\nContent-Length: %s\r\n\r\n%s\r\n\r\n"
HTTP_DISCONNECT_TIMEOUT = 30 # half a minute timeout
CATEGORY = "AIRi-Stream"

class MultiPartStream():
	BOUNDARY = 'myBOUNDARY'
	clients = []
	request = None
	oneshot = False

	@report(category=CATEGORY)
	def __init__(self, request):
		self.request = request
		self.request.connectionLost = self.connectionLost
		self.request.multipart=self
		self.oneshot = self.request.args.get('oneshot', ['false',])[0].lower() == 'true'
		self.thumbnail = self.request.args.get('thumbnail', ['false',])[0].lower() == 'true'
		self.flash = 'flash' in self.request.args
		MultiPartStream.clients.append(self)

	@classmethod
	def getBoundary(klass):
		return "--%s\r\n" % (klass.BOUNDARY)

	def writeBoundary(self):
		self.request.write(MultiPartStream.getBoundary())

	def writeStop(self):
		self.request.write("%s--\r\n" % MultiPartStream.getBoundary())

	@report(category=CATEGORY)
	def process(self):
		self.request.setHeader('Cache-Control', 'no-cache, no-store, must-revalidate;')
		self.request.setHeader('Expires', '0');
		self.request.setHeader('Pragma-directive', 'no-cache')
		self.request.setHeader('Pragma', 'no-cache')
		self.request.setHeader('Cache-directive', 'no-cache')
		if not (self.oneshot or self.thumbnail):
			self.request.setHeader('Connection', 'Keep-Alive')
			if self.flash:
				#self.request.setHeader('Content-Type', 
				#	'multipart/x-mixed-replace')
				self.request.setHeader('Content-Type',
					'text/html;')
			else:
				self.request.setHeader('Content-Type', 
					'multipart/x-mixed-replace;boundary=%s' % (self.BOUNDARY))

	@report(category=CATEGORY)
	def connectionTimeout(self):
		'''Called after X seconds after the http stream is closed'''
		if len(MultiPartStream.getClients(self.target)):
			return
		log.msg("No more clients, closing link")
		CameraFactory.disconnect(self.target)
		del self
  
	@report(category=CATEGORY)
	def connectionLost(self, reason):
		'''Called when the http stream is closed'''
		if self in MultiPartStream.clients:
			MultiPartStream.clients.remove(self)
		log.msg("Registering to disconnect in %s seconds" % HTTP_DISCONNECT_TIMEOUT)
		reactor.callLater(HTTP_DISCONNECT_TIMEOUT, 
			MultiPartStream.connectionTimeout, self)
	
	@report(category=CATEGORY)
	def finish(self):
		'''Gets called by oneshot or thumbnail modes''' 
		try:
			self.request.finish()
		except Exception, err:
			log.err(err)
		MultiPartStream.clients.remove(self)

	def sendPart(self, content, mime="text/html", MULTIPART=None):
		if self.oneshot or self.thumbnail:
			try:
				self.request.setHeader("Content-Type", mime)
				self.request.setHeader("Content-Size", len(content))
				self.request.write(content)
				self.request.write("\n\n")
				self.finish()
			except Exception, err:
				log.msg("error during sendPart")
				log.err(err)
		else:
			if MULTIPART is None:
				MULTIPART=MULTIPARTRESPONSE % (MultiPartStream.BOUNDARY, mime, len(content), content)
			try:
				self.request.write(MULTIPART)
			except Exception, err:
				log.msg("error during sendPart")
				log.err(err)

	@classmethod
	def sendToClients(klass, content, mime="text/html"):
		if len(klass.clients) == 0:
			return

		size = len(content)
		out=MULTIPARTRESPONSE % (klass.BOUNDARY, mime, size, content)

		for client in klass.clients:
			client.sendPart(content, mime, out)

#		log.msg("sendToClients, size: %s, count: %s" %(len(content), len(klass.clients)), 
#			loglevel=DEBUG,
#			category="MultiPartStream")

	@classmethod
	def getClients(klass, address):
		def internal():
			for c in klass.clients:
				if c.target.lower() == address:
					yield c
		address = address.lower()
		return list(internal())

class StreamResource(Resource, Listener):
	isLeaf = True
  
	@classmethod
	def getClients(klass, address):
		return MultiPartStream.getClients(address)

	def gotFrame(self, frame, address):
		#print "StreamResource.gotFrame %s" % len(frame)
		size = len(frame)
		out=MULTIPARTRESPONSE % (MultiPartStream.BOUNDARY, "image/jpeg", size, frame)

		for client in StreamResource.getClients(address):
			client.sendPart(frame, "image/jpeg")

	@report(category=CATEGORY)
	def lostConnection(self, reason, failed, address):
		'''Called when the Bluetooth Link is lost'''
		print "StreamResource.lostConnection", address
		clients = StreamResource.getClients(address)
		for c in clients:
			c.sendPart(str(reason))
			if not c.oneshot:
				try:
					c.request.finish()
				except Exception, err:
					log.err(err)
			MultiPartStream.clients.remove(c)

	@report(category=CATEGORY, level=RESULT)
	def render_GET(self, request):
		address = request.path.split("/",2)[-1].replace("_", ":")
		request.transport.socket.settimeout(5)
		multipart = MultiPartStream(request)
		multipart.process()
		multipart.target = address
		if len(address) == 17:
			ready = CameraFactory.isConnected(address) or CameraFactory.isPending(address)
			if multipart.thumbnail and not ready:
				multipart.sendPart("")
				return server.NOT_DONE_YET
			if not ready:
				method = request.args.get("method", ["RFCOMM",])[-1]
				try:
					CameraFactory.connect(address, 1, method)
				except Exception, err:
					log.msg("Failed while trying to connect")
					log.err(err)
		CameraFactory.registerListener(address, self)
		return server.NOT_DONE_YET

class TestStreamResource(Resource):
	isLeaf = True

	def render_GET(self, request):
		log.msg(str(request))
		log.msg(str(request.requestHeaders))
		multipart = MultiPartStream(request)
		multipart.process()
		return server.NOT_DONE_YET

	def render_POST(self, request):
		return self.render_GET(request)

if __name__ == '__main__':
	import webcam
	from twisted.web.server import Site
	import sys
	log.startLogging(sys.stdout)

	webcam.init(False)
	def sendFrame():
		MultiPartStream.sendToClients(webcam.repeat().tostring(), "image/jpeg")
		reactor.callLater(1/15., sendFrame)

	root = Resource()
	root.isLeaf = False
	root.putChild("stream", TestStreamResource())
	reactor.listenTCP(8800, Site(root), interface="0.0.0.0")
	reactor.callLater(0, sendFrame)
	reactor.run()#!/usr/bin/env python




from twisted.web.resource import Resource
from twisted.web import http, server
from twisted.python import log
from twisted.internet import threads, reactor
from time import time
import json
import twisted_bluetooth as bluetooth
import traceback
from airi.camera.protocol import CameraFactory
from airi.settings import getSettings
from airi import report

settings = getSettings()

SCAN_TIMEOUT=60 # scan is started once each 60 seconds

class ConnectionTest():
  def __init__(self, req):
    self.request = req

  def cleanup(self, address):
    CameraFactory.disconnect(address)
    CameraFactory.removeListener(address, self)
    self.request.finish()

  def lostConnection(self, reason, failed, address):
    print "ConnectionTest.lostConnection", address
    self.request.write("<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % reason)
    self.request.setResponseCode(500, str(failed))
    self.request.finish()
    self.cleanup(address)

  def gotFrame(self, frame, address):
    print "ConnectionTest.gotFrame", address
    cli = CameraFactory.getCamera(address)
    self.request.write(json.dumps(cli))
    self.cleanup(address)

class DevicesManager(Resource):
  isLeaf = True

  def render_GET(self, request):
    request.setHeader('Content-Type', 'application/json')
    request.setHeader('Cache-Control', 'no-cache')
    request.setHeader('Connection', 'close')
    return json.dumps(list(CameraFactory.getCameras()))

class ConfigurationManager(Resource):
  isLeaf = True

  def render_GET(self, request):
    try:
      request.setHeader('Content-Type', 'application/json')
      request.setHeader('Cache-Control', 'no-cache')
      request.setHeader('Connection', 'close')
      if len(request.postpath) > 0 and len(request.postpath[0])>0:
        address=request.postpath[0].replace("_", ":")
        if len(request.args) == 0:
          return json.dumps(CameraFactory.getCamera(address))
        camera = settings.getCamera(address)
        for key in request.args:
            settings.setCameraSetting(address, key, request.args[key][-1])
        settings.save()
        cli = CameraFactory.getConnected(address)
        if cli:
          cli.updateSettings()
        return json.dumps(CameraFactory.getCamera(address))
      request.setResponseCode(500, "invalid address")
      return "Invalid address"

    except Exception, err:
      print err
      CameraFactory.connect(address, 1, "RFCOMM")
      CameraFactory.registerListener(address, ConnectionTest(request))
      return server.NOT_DONE_YET

  def render_POST(self, request):
    out = {}
    print str(request.args)
    for key, value in request.args.iteritems():
      if len(value) > 1:
        out[key] = True
      else:
        out[key]=value[0]
    settings.setCamera(out)
    print out
    settings.save()
    cli = CameraFactory.getConnected(out['address'])
    if cli:
      cli.updateSettings()
    return "saved"

class doConfiguration(Resource):
  isLeaf = True

  def render_POST(self, request):
    address = request.args["address"][0]
    option = request.args["option"][0]
    value = request.args["value"][0]
    if value.lower() in ["true", "false"]:
	value = value.lower()=="true"
    log.msg("doConfiguration.POST(%s, %s, %s)" % (address, option, value))
    try:
      cli = CameraFactory.getConnected(address)
      if cli:
        cli.client.set(option, value)
      return "updated"
    except Exception, err:
      return "failed: %s" % err

class ConnectionManager(Resource):
  isLeaf = True

  def render_all(self, request):
    out = {}
    for addr, cli in CameraFactory.getConnected().iteritems():
      out[addr] = CameraFactory.getCamera(addr)
    return out

  def render_GET(self, request):
    try:
      request.setHeader('Content-Type', 'application/json')
      request.setHeader('Cache-Control', 'no-cache')
      request.setHeader('Connection', 'close')
      if len(request.postpath) > 0 and len(request.postpath[0])>0:
        address=request.postpath[0].replace("_", ":")
        cli = CameraFactory.getCamera(address, silent=True)
        if cli or not request.args.get("test", None):
          return json.dumps(CameraFactory.getCamera(address))
        CameraFactory.connect(address, 1, "RFCOMM")
        CameraFactory.registerListener(address, ConnectionTest(request))
        return server.NOT_DONE_YET
      else:
        return json.dumps(self.render_all(request))
    except Exception, err:
      request.setHeader('Content-Type', 'text/html')
      request.setHeader('Cache-Control', 'no-cache')
      request.setHeader('Connection', 'close')
      request.setResponseCode(500, str(err))
      return "<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % (traceback.format_exc())

class ScanManager(Resource):
  isLeaf = True
  lastscan = None
  scancache = None
  scanning = None
  waiting = []

  def do_work(self):
    error = None
    try:
      log.msg("Doing scan")
      cache = bluetooth.discover_devices(lookup_names=True)
      log.msg(cache)
      self.scancache = [{
          'address':x[0], 
          'name':x[1], 
          'state': CameraFactory.isConnected(x[0])}
          for x in cache]
      log.msg(self.scancache)
      remove = []
      print self.waiting
      for request in self.waiting:
        request.setHeader('Content-Type', 'application/json')
        request.setHeader('Cache-Control', 'no-cache')
        request.setHeader('Connection', 'close')
        request.write(json.dumps(self.scancache))
        request.finish()
      if len(self.waiting)==0:
        return server.NOT_DONE_YET
    except Exception, err:
      log.err(err)
      error = err

    try:
      error = "<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % traceback.format_exc()
      for request in self.waiting:
        if request.finished:
          continue
        request.setHeader('Content-Type', 'text/html')
        request.setHeader('Cache-Control', 'no-cache')
        request.setHeader('Connection', 'close')
        request.setResponseCode(500, error)
        request.write("<html><h1>ERROR:</h1>\n")
        request.write("<pre>%s</pre></html>" % traceback.format_exc())
        request.finish()
    except Exception, err:
      log.err(err)
    self.waiting = []
    self.lastscan = None
    log.msg("Scan thread done")
    return server.NOT_DONE_YET


  def lost_connection(self, err, request):
    log.msg("lost_connection(%s)" % str(request))
    self.waiting.remove(request)
  
  def render_GET(self, request):
    if self.lastscan and time()-self.lastscan < SCAN_TIMEOUT:
      if not self.scancache:
        self.waiting.append(request)
        request.notifyFinish().addErrback(self.lost_connection, request)
        log.msg("%s" % self.waiting)
        log.msg("Tried to start a parallel scan, we wait")
        return server.NOT_DONE_YET

      request.setHeader('Content-Type', 'application/json')
      request.setHeader('Cache-Control', 'no-cache')
      request.setHeader('Connection', 'close')
      request.write(json.dumps(self.scancache))
      request.finish()
      log.msg("Scan return from cache")
      return server.NOT_DONE_YET

    self.lastscan = time()
    self.waiting.append(request)
    request.notifyFinish().addErrback(self.lost_connection, request)
    log.msg("%s" % self.waiting)
    log.msg("Starting scan thread")
    threads.deferToThread(self.do_work)
    return server.NOT_DONE_YET

class DisconnectManager(Resource):
  isLeaf = True

  def render_GET(self, request):
    try:
      request.setHeader('Content-Type', 'application/json')
      request.setHeader('Cache-Control', 'no-cache')
      request.setHeader('Connection', 'close')
      if len(request.postpath) > 0 and len(request.postpath[0])>0:
        address=request.postpath[0].replace("_", ":")
        if not CameraFactory.isConnected(address):
          return json.dumps({"result": "Not Connected"})
        CameraFactory.disconnect(address)
        return json.dumps({"result": "Disconnected"})
      return json.dumps({"result": "No Address"})
    except Exception, err:
      request.setHeader('Content-Type', 'text/html')
      request.setHeader('Cache-Control', 'no-cache')
      request.setHeader('Connection', 'close')
      request.setResponseCode(500, str(err))
      return "<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % (traceback.format_exc())

class StateManager(Resource):
  isLeaf = True

  def render_POST(self, request):
    try:
      request.setHeader('Content-Type', 'application/json')
      request.setHeader('Cache-Control', 'no-cache')
      request.setHeader('Connection', 'close')
      address = request.args["address"][0]
      if CameraFactory.isConnected(address):
        CameraFactory.disconnect(address)
        return json.dumps({"result": "disconnected"})
      else:
        CameraFactory.connect(address)
        return json.dumps({"result": "connecting"})
    except Exception, err:
      request.setHeader('Content-Type', 'text/html')
      request.setHeader('Cache-Control', 'no-cache')
      request.setHeader('Connection', 'close')
      request.setResponseCode(500, str(err))
      return "<html><h1>ERROR:</h1>\n<pre>%s</pre></html>" % (traceback.format_exc())

class UpdateManager(Resource):
  isLeaf = True
  clients = {}
  reference = None

  @report(category="UpdateManager")
  def __init__(self):
    UpdateManager.reference=self

  @report("UpdateManager")
  def timeout(self, connection, address):
    log.msg("UpdateManager.timeout (%s, %s)" % ( connection, address ))
    connection.write(json.dumps({"result": "timeout"}))
    connection.finish()
    if address in UpdateManager.reference.clients:
      if connection in UpdateManager.reference.clients[address]:
        UpdateManager.reference.clients[address].remove(connection)

  @classmethod
  @report(category="UpdateManager")
  def propagate(klass, address, args):
    if not klass.reference:
      return

    if not address in klass.reference.clients:
      return

    if "address" not in args:
      args["address"] = address

    for r in klass.reference.clients[address]:
      try:
        r.write(json.dumps(args))
        r.finish()
        r.delayed.cancel()
      except Exception, err:
        log.err(err)
    klass.reference.clients.pop(address)

  @report(category="UpdateManager")
  def lost_connection(self, err, address, request):
    log.msg("updateManager.lost_connection(%s, %s)" % (address, request))
    request.delayed.cancel()
    if address in self.clients:
      if request in self.clients[address]:
        self.clients[address].remove(request)

  @report(category="UpdateManager")
  def render_POST(self, request):
    address = request.args["address"][0]
    if address not in self.clients:
      self.clients[address] = []
    self.clients[address].append(request)
    request.delayed = reactor.callLater(60, self.timeout, connection=request, address=address)
#    request.delayed.addErrback(log.err)
    request.notifyFinish().addErrback(self.lost_connection, address=address, request=request)
    request.setHeader('Content-Type', 'application/json')
    request.setHeader('Cache-Control', 'no-cache')
    request.setHeader('Connection', 'close')
    return server.NOT_DONE_YET

class API_Root(Resource):
  isLeaf = True

  def render_GET(self, request):
    return "Invalid"

class API(Resource):
  isLeaf = False

  def __init__(self, *a, **kw):
    log.msg("API()")
    Resource.__init__(self, *a, **kw)
    self.putChild("", API_Root())
    self.putChild("scan", ScanManager())
    self.putChild("devices", DevicesManager())
    self.putChild("connected", ConnectionManager())
    self.putChild("configure", ConfigurationManager())
    self.putChild("doconfigure", doConfiguration())
    self.putChild("disconnect", DisconnectManager())
    self.putChild("updates", UpdateManager())
    self.putChild("switchstate", StateManager())

if __name__=='__main__':
  from twisted.application.service import Application
  from twisted.application.internet import TCPServer
  from twisted.web.server import Site
  from twisted.internet import reactor
  import sys
  log.startLogging(sys.stdout)

  root = Resource()
  root.putChild("api", API())
  reactor.listenTCP(8800, Site(root), interface="0.0.0.0")
  reactor.run()#!/usr/bin/env python



# -*- coding: utf-8 -*-
try:
    from twisted.internet import glib2reactor
    glib2reactor.install()
except:
    pass

from twisted.python import log
import logging

ENTRY   = 10
ARGS    = 20
RESULT  = 30

class report:
  """
  Decorator that prints information about function calls.
  Based on: http://paulbutler.org/archives/python-debugging-with-decorators/
  """

  def __init__(self, level=ENTRY, category=None):
    self.level=level
    self.category = category

  def __call__(self, fn):
    def wrap(*args, **kwargs):
      if self.level >= ARGS:
        fc="call to %s.%s (%s,%s)" % (
          fn.__module__, fn.__name__,
          ', '.join( [ repr(a) for a in args ] ),
          ', '.join( ["%s = %s" % (a, repr(b)) for a,b in kwargs.items()] )
        )
        log.msg(fc, level=logging.DEBUG, category=self.category)
      elif self.level >= ENTRY:
        log.msg("call to %s.%s" % (fn.__module__, fn.__name__), category=self.category, level=logging.DEBUG)

      ret = fn(*args, **kwargs)
      if self.level >= RESULT:
        log.msg("%s returned %s" % (fc, ret), level=logging.DEBUG)
      return ret
    if self.level > 0:
      return wrap
    return fn

__version__ = ('1', '0', 'c5')
__version__ = '.'.join(__version__)

import camera
from camera import dbg, CameraProtocol
import settings
import twisted_bluetooth
import stream
import server
import api



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from airi.jinja import main as jmain
from twisted.internet import reactor
from twisted.python import log

def main():
	import sys
	if sys.platform == "linux2" and "-nopairing" not in sys.argv:
		from pair import main
		reactor.callWhenRunning(main)
	log.startLogging(sys.stdout)
	if len(sys.argv) > 1:
		jmain(int(sys.argv[1]))
	else:
		jmain()
	reactor.run()

if __name__ == '__main__':
	main()




# -*- coding: utf-8 -*-
import cv

global capture, show
show = False
capture = None

def init(show_=False):
  global capture, show, camera_index
  if show_:
    cv.NamedWindow("w1", cv.CV_WINDOW_AUTOSIZE)
  show = show_
  camera_index = 0
  capture = cv.CaptureFromCAM(camera_index)

def repeat():
  global capture #declare as globals since we are assigning to them now
  global camera_index
  global show
  frame = cv.QueryFrame(capture)
  if show:
	cv.ShowImage("w1", frame)
  c = cv.WaitKey(50)
  if c>-1 and chr(c) =='q':
    print chr(c)
    return None
  return cv.EncodeImage(".jpeg", frame, [cv.CV_IMWRITE_JPEG_QUALITY, 50])

if __name__=='__main__':
  init(True)
  while repeat():
	continue



# -*- coding: utf-8 -*-
from twisted.web.resource import Resource
from twisted.python import log
from twisted.web.static import File
from jinja2 import Environment, PackageLoader
from airi import __version__
from airi.camera.protocol import CameraFactory
from airi.camera import UnknownDevice
from airi.settings import getSettings
from airi.twisted_bluetooth import resolve_name
from airi.stream import StreamResource
from airi.sco import SCOResource
import airi.twisted_bluetooth as bluetooth
import pkg_resources, os, time

settings = getSettings()

class TemplateResource():
  def __init__(self, template, context):
    self.template = template
    self.context = context

  def render(self, *args, **kwargs):
    return str(self.template.render(self.context))

class Main(Resource):
  isLeaf = False

  def __init__(self, home="/"):
    Resource.__init__(self)
    self.env = Environment(loader=PackageLoader("airi", "templates"))
    self.env.globals["home"]=home

  def index(self, request):
    devices = list(CameraFactory.getCameras())
    for dev in devices:
        dev["viewers"] = StreamResource.getClients(dev["address"])
    return {
      "devices": devices 
    }

  def setup(self, request):
      def saveCamera(args):
          if "enable_pincode" in args:
            if args["enable_pincode"][0].lower()=="true":
              settings.setPIN(block=args["address"][0],
                          npin=args["pincode"][0])
            else:
              settings.delPIN(args["address"][0])
          settings.save()
          for k in ["save", "last", "battery", "status",
                            "enable_pincode", "pincode", "new-device"]:
              if k in args:
                  args.pop(k)
          for k in args.keys():
              args[k]=args[k][0]
          print "saving camera", args
          settings.setCamera(args)
          settings.save()

      def deleteCamera(address):
          settings.deleteCamera(address)
          settings.save()

      def getCamera(address):
          out = {}
          try:
              out.update(CameraFactory.getCamera(address))
          except UnknownDevice, err:
              out["address"] = address
              out["name"] = resolve_name(address)
              out["new_device"] = True
              out["types"] = CameraFactory.getTypes()
          print out
          return out

      if "address" not in request.args:
          raise Exception("Invalid address")

      address = request.args["address"][-1]

      if request.method == "POST":
          if "save" in request.args:
              saveCamera(request.args)
          if "delete" in request.args:
              deleteCamera(address)
      return getCamera(address)

  def scan(self, request):
    try:
      log.msg("Doing scan")
      cache = bluetooth.discover_devices(lookup_names=True)
      log.msg(cache)
      out = {"devices":
                [{
                  'address':x[0],
                  'name':x[1],
                  'state': CameraFactory.isConnected(x[0])
                }
                    for x in cache
                ]
            }
      log.msg(out)
      return out
    except Exception, err:
      log.err(err)
      return {"error": str(err)}

  def server_setup(self, request):
	if request.method == "POST":
		if "delete" in request.args:
			settings.delPIN(request.args["delete"][0])
			settings.save()
		elif "save" in request.args:
			request.args.pop("save")
			blocks = dict([ (b,request.args[b][0]) for b in request.args if b.startswith("block") ])
			pins = dict([ (b,request.args[b][0]) for b in request.args if b.startswith("value") ])
			sets = {}
			for b in blocks:
				if len(blocks[b].strip()) == 0:
					continue
				settings.setPIN(block=blocks[b], npin=pins["value"+b.replace("block","")])
			settings.save()

	out = {}
	out["pins"]=settings.getPINs()
	return out

  def stream(self, request):
    if "address" not in request.args:
      raise Exception("You need to provide with an address")
    camera = CameraFactory.getCamera(request.args["address"][0], True)
    if camera["enable_pincode"]:
	camera["pincode"]="************"
    print camera
    return {
      "isChrome": "chrome" in request.requestHeaders.getRawHeaders("user-agent")[0].lower(),
      "address": request.args["address"][0],
      "camera": camera
    }

  contexts = {
    "index.html": index,
    "setup.html": setup,
    "scan.html": scan,
    "server-setup.html": server_setup,
    "stream.html": stream
  }

  def getChild(self, path, request):
	if "media" in path or "favicon.ico" in path:
		return Resource.getChild(self, path, request)
	if len(path) == 0:
		path = "redirect.html"
	template = self.env.get_template(path)
	context = self.contexts.get(path, lambda x,y: {})(self, request)
	context["pairing_supported"]=bluetooth.isPairingSupported()
	context["version"] = __version__
	return TemplateResource(template, context)

class PkgFile(File):
    def __init__(self, path, *args, **kwargs):
        File.__init__(self, path, *args, **kwargs)
        self.path = path

    def isdir(self):
        out = pkg_resources.resource_isdir("airi", self.path)
        return out

    def exists(self):
        out = pkg_resources.resource_exists("airi", self.path)
        return out

    def isfile(self):
        out = not self.isdir() and self.exists()
        return out

    def child(self, path):
        path_ = os.path.join(self.path, path)
        def exists():
            return pkg_resources.resource_exists("airi", path_)
        o = self.clonePath(path_)
        o.exists = exists
        o.path = path_
        return o

    def __getresource(self):
        if not self.__resource:
            self.__resource = pkg_resources.resource_stream("airi", self.path)
        return self.__resource

    def getmtime(self):
        return time.time()

    def openForReading(self): 
        return pkg_resources.resource_stream("airi", self.path)

    def getsize(self):
        return len(pkg_resources.resource_stream("airi", self.path).read())

    def getChild(self, path, request):
        return File.getChild(self, path, request)

def main(port=8000):
    from twisted.application.service import Application
    from twisted.application.internet import TCPServer
    from twisted.web.server import Site
    from twisted.internet import reactor
    from airi.api import API
    import sys
    log.startLogging(sys.stdout)

    root = Main()
    path = os.path.dirname(os.path.realpath(__file__))
    root.putChild("api",        API())
    root.putChild("media", PkgFile("/media"))#os.path.join(path, "media/")))
    root.putChild("sco", SCOResource())
    root.putChild("stream", StreamResource())
    p=reactor.listenTCP(port, Site(root), interface="0.0.0.0", backlog=5)
    return p

if __name__=='__main__':
    from twisted.internet import reactor
    main()
    reactor.run()




# -*- coding: utf-8 -*-
# Copyright (c) 2011 Naranjo Manuel Francisco <manuel@aircable.net>
# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
# See Twisted LICENSE for details.

"""
Various Bluetooth Socket classes
"""

# System Imports
import os
import types
import sys
import operator
import lightblue as bluetooth
from lightblue import *
import socket

from twisted.internet import tcp
from twisted.internet.tcp import *

from twisted.python import log

from airi import report

CATEGORY="darwin"

from multiprocessing.connection import Listener
def isPairingSupported():
	return False

@report(category=CATEGORY)
def discover_devices (duration=8, flush_cache=True, lookup_names=False):
    out = bluetooth.finddevices(getnames=lookup_names, length=duration)
    print out
    return [ (o[0], o[1]) for o in out ]

class BluetoothConnection(tcp.Connection):
    """
    Superclass of all Bluetooth-socket-based Descriptors

    This is an abstract superclass of all objects which represent a Bluetooth
    connection based socket.

    @ivar logstr: prefix used when logging events related to this connection.
    @type logstr: C{str}

    """

class ConnectionWrapper():
    def __init__(self, connection):
        self.__conn = connection
        for attr in dir(connection):
            if not attr.startswith("__"):
                if not getattr(self, attr, None):
                    setattr(self, attr, getattr(connection, attr))

    def send(self, what):
        if type(what) == buffer:
            what = str(buffer)
        self.__conn.send(what)
        return len(what)

    @report(category=CATEGORY)
    def bind(self, *args, **kwargs):
        pass

    @report(category=CATEGORY)
    def shutdown(self, *args, **kwargs):
        self.__conn.close()

    @report(category=CATEGORY)
    def setblocking(self, *args, **kwargs):
        pass

class BluetoothBaseClient(tcp.Connection):
    """A base class for client Bluetooth (and similiar) sockets.
    """
    addressFamily = 31 # AF_BLUETOOTH
    proto = None

    @report(category=CATEGORY)
    def createInternetSocket(self):
        """(internal) Create a non-blocking socket using
        self.addressFamily, self.socketType.
        """
        if self.proto not in [ None, bluetooth.RFCOMM, bluetooth.L2CAP ]:
            raise RuntimeException("I only handle bluetooth sockets")

        self.listener = Listener()
        pid = os.fork()
        if pid == 0:
            # I'm the child
            print "Child running"
            os.execvp(sys.executable, [sys.executable, "-m", "airi.twisted_bluetooth.darwin.helper",
                self.listener.address])
            # and we lost it
            print "OOOOOOOOOOOOOOOOOPPPPPPSSSSSSSS!!!!!!!!!"

        print "Luke I'm your father"
        self.childpid = pid
        s = self.listener.accept()
        return ConnectionWrapper(s)

    @report(category=CATEGORY)
    def resolveAddress(self):
        self.realAddress = self.addr
        print "resolveAddress", self.realAddress
        self.doConnect()

    @report(category=CATEGORY)
    def doConnect(self):
        """I connect the socket.

        Then, call the protocol's makeConnection, and start waiting for data.
        """
        if not hasattr(self, "connector"):
            # this happens when connection failed but doConnect
            # was scheduled via a callLater in self._finishInit
            return

        self.socket.send([self.proto, self.realAddress])
        err = self.socket.recv()
        if err:
            self.socket.close()
            self.listener.close()
            os.waitpid(self.childpid, 0)
            self.failIfNotConnected(err)
            return

        # If I have reached this point without raising or returning, that means
        # that the socket is connected.
        del self.doWrite
        del self.doRead
        # we first stop and then start, to reset any references to the old doRead
        self.stopReading()
        self.stopWriting()
        self._connectDone()

    @report(category=CATEGORY)
    def fileno(self):
        return self.socket.fileno()

    def doRead(self):
        """Calls self.protocol.dataReceived with all available data.

        This reads up to self.bufferSize bytes of data from its socket, then
        calls self.dataReceived(data) to process it.  If the connection is not
        lost through an error in the physical recv(), this function will return
        the result of the dataReceived call.
        """
        data = self.socket.recv()
        if type(data) != str:
            return main.CONNECTION_LOST
        return self.protocol.dataReceived(data)

class Client(BluetoothBaseClient, tcp.Client):
    """A Bluetooth client."""

    @report(category=CATEGORY)
    def __init__(self, proto, *args, **kwargs):
        if proto not in [ None, bluetooth.RFCOMM, bluetooth.L2CAP ]:
            raise RuntimeException("I only handle bluetooth sockets")
        self.proto = proto
        super(Client, self).__init__(*args, **kwargs)

    @report(category=CATEGORY)
    def getHost(self):
        """Returns a Bluetooth address.

        This indicates the address from which I am connecting.
        """
        return self.socket.getsockname()

    @report(category=CATEGORY)
    def getPeer(self):
        """Returns a Bluetooth address.

        This indicates the address that I am connected to.
        """
        return self.realAddress

class Server(BluetoothBaseClient, tcp.Connection):
    """
    Serverside bluetooth socket-stream connection class.

    This is a serverside network connection transport; a socket which came from
    an accept() on a server.
    """

    @report(category=CATEGORY)
    def startTLS(self, ctx, server=1):
        raise RuntimeException("not valid method")

    @report(category=CATEGORY)
    def getHost(self):
        """Returns a Bluetooth address.

        This indicates the server's address.
        """
        return self.socket.getsockname()

    @report(category=CATEGORY)
    def getPeer(self):
        """Returns a Bluetooth address.

        This indicates the client's address.
        """
        return self.client

class Port(tcp.Port):
    """
    A Bluetooth server port, listening for connections.

    L{twisted.internet.tcp.Port}
    """
    addressFamily = 31 # socket.AF_BLUETOOTH
    proto = None

    @report(category=CATEGORY)
    def __init__(self, proto, port, factory, backlog=50, interface='', reactor=None):
        """Initialize with a numeric port to listen on.
        """
        tcp.Port.__init__(self, port, factory, backlog, interface, reactor)
        if proto not in [ None, bluetooth.RFCOMM, bluetooth.L2CAP ]:
            raise RuntimeException("I only do Bluetooth")
        self.proto = proto

    @report(category=CATEGORY)
    def createInternetSocket(self):
        s = bluetooth.BluetoothSocket(self.proto)
        if platformType == "posix" and sys.platform != "cygwin":
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        return s

    @report(category=CATEGORY)
    def _buildAddr(self, (host, port)):
        return (host, port)

    @report(category=CATEGORY)
    def getHost(self):
        """Returns a Bluetooth Address

        This indicates the server's address.
        """
        return self.socket.getsockname()

class Connector(tcp.Connector):
    @report(category=CATEGORY)
    def __init__(self, proto, host, port, *a, **kw):
        tcp.Connector.__init__(self, host, int(port), *a, **kw)
        self.proto = proto

    @report(category=CATEGORY)
    def _makeTransport(self):
        return Client(self.proto, self.host, self.port, self.bindAddress, self, self.reactor)

    @report(category=CATEGORY)
    def getDestination(self):
        return (self.host, self.port)

@report(category=CATEGORY)
def __connectGeneric(reactor, proto, host, port, factory, timeout=30, bindAddress=None):
    c = Connector(proto, host, port, factory, timeout, bindAddress, reactor)
    c.connect()
    return c

@report(category=CATEGORY)
def connectRFCOMM(reactor, *a, **kw):
    return __connectGeneric(reactor, bluetooth.RFCOMM, *a, **kw)

@report(category=CATEGORY)
def connectL2CAP(reactor, *a, **kw):
    return __connectGeneric(reactor, bluetooth.L2CAP, *a, **kw)

@report(category=CATEGORY)
def resolve_name(address):
    try:
        return bluetooth.finddevicename(address, True)
    except Exception, err:
        print err
    return None





# -*- coding: utf-8 -*-
from twisted.python import log
from twisted.internet import reactor
from airi.camera import dbg, CameraProtocol
from airi.settings import getSettings


settings=getSettings()

SIZES={
    "VGA":  4,
    "QVGA": 5,
    "QQVGA":6,
    "SVGA": 0xc,
    "XVGA": 0xd,
}

CAPABILITIES={ 
  'size':       SIZES.keys(),
  'pan':        [],
  'flash':      False,
  'voice':      False,
  'exposure':   False,
  'transport':  ['RFCOMM',],
  'battery':    False,
}

IDLE=0
ECHO=1
PREVIEW=2
COMMAND_MODE=3

class OptiEye(CameraProtocol):
  client = None
  state = IDLE
  callLater = None
  size = "QVGA"

  def getType(self):
    return 'OptiEye'

  def getCapabilities(self):
    return CAPABILITIES

  def getTransport(self):
    return "RFCOMM"

  def __init__(self, client):
    self.client = client
    self.address = client.address
    self.transport = client.transport
    self.doCommandMode()

  def setSize(self, size):
    if size not in SIZES:
      raise RuntimeException("Size not valid")
    self.size = size

  def getSize(self):
    return self.size

  def getSizes(self):
    return SIZES

  def __doCommand(self, command):
    self.transport.write("$GENIESYS%04X\r\n" % command)

  def updateSettings(self):
    self.doCommandMode()
    
  def __cancelLater(self):
    if not self.callLater:
      return
    if self.callLater.active():
      self.callLater.cancel()
    self.callLater = None

  def doCommandMode(self):
    dbg("doCommandMode")
    self.state = COMMAND_MODE
    self.__doCommand(1)
    self.__cancelLater()
    self.callLater = reactor.callLater(1, self.doSetSize)
    #self.callLater.addErrback(log.err)

  def doSetSize(self, size=None):
    self.state = ECHO
    self.__cancelLater()
    if not size:
      camera=settings.getCamera(self.address)
      size = "QVGA" #default
      if camera:
        size = camera.get("size", size)
    dbg("doSetSize(%s)", size)
    self.__doCommand(SIZES[size])
    self.callLater = reactor.callLater(2, self.doPreview)
    #self.callLater.addErrback(log.err)

  def doPreview(self):
    dbg("doPreview")
    self.state = PREVIEW
    self.__cancelLater()
    self.callLater = reactor.callLater(20, self.transport.loseConnection)
    #self.callLater.addErrback(log.err)
    self.__doCommand(2)

  def previewData(self):
    #dbg("previewData")
    self.callLater.reset(20)
    start = self.client.buffer.find("\xff\xd8")
    end = self.client.buffer.find("\xff\xd9")
    if start == -1 or end == -1:
      return

    frame = self.client.buffer[start:end+2]
    self.client.buffer = self.client.buffer[end+2:]
    self.client.gotFrame(frame)

  def dataReceived(self):
    #dbg("OptiEye.dataReceived, have %s bytes" % len(self.client.buffer))
    #dbg("state = %s" % self.state)

    if self.state == COMMAND_MODE:
      self.callLater.reset(1) # wait another second
      #dbg("Dropping %s" % self.client.buffer)
      self.client.buffer = ""
      return

    if self.state == PREVIEW:
      return self.previewData()

  def set(self, option, value):
    dbg("set %s->%s" % (option, value) )
    if option=="size":
        self.doSetSize(value)
        from airi.api import UpdateManager
        UpdateManager.propagate(self.address, {"size": value})
    else:
        dbg("ignored")

  def disconnect(self):
    dbg("OptiEyes.disconnect")
    self.__doCommand(1)

OptiEye.Capabilities = CAPABILITIES
OptiEye.Sizes = SIZES
OptiEye.Panning = None



# -*- coding: utf-8 -*-
import ConfigParser
from os import path, access, W_OK, environ
from twisted.python import log
import logging, sys
from airi import report

DEFAULTS={
  "dongles": {
    "default": True,
  },
  "pin": {
    "default": 1234,
  }
}

class Settings():
  instance = None

  def __init__(self, name=None):
    if not name:
        parent = ""
        if access(path.dirname(path.realpath(__file__)), W_OK):
            parent = path.dirname(path.realpath(__file__))
        else:
            if "HOME" in environ:
                parent = environ["HOME"]
            elif "HOMEPATH" in environ:
                parent = environ["HOMEPATH"]
            elif "DATA_PATH" in environ:
                parent = environ["DATA_PATH"]
        if not sys.platform.startswith("win"):
            name=path.join(parent, ".AIRi")
        else:
            name=path.join(parent, "AIRi.cfg")
    print "configuration file", name
    self.name = name
    self.reload()


  def reload(self):
    self.config = ConfigParser.SafeConfigParser()
    for section in DEFAULTS:
      if self._has_section(section):
        continue;

      self._add_section(section)
      for key, value in DEFAULTS[section].iteritems():
        self._set(section, key, str(value))

    if path.exists(self.name) and path.isfile(self.name):
      print "loading from", self.name
      self._read(self.name)

  # @report()
  def getDongle(self, address):
    self.reload()
    address = address.replace(":", "_")
    dongles = self._options("dongles")
    for op in dongles:
      if address.lower().startswith(op):
        return self._getboolean("dongles", op)
    return self._getboolean("dongles", "default")

  # @report()
  def getCameraSections(self):
    self.reload()
    cameras = self._sections()
    if "dongles" in cameras:
      cameras.remove("dongles")
    if "pin" in cameras:
      cameras.remove("pin")
    return cameras

  # @report()
  def getCameraSection(self, address, create=False):
    self.reload()
    cameras = self.getCameraSections()
    for cam in cameras:
      if address.lower().startswith(self._get(cam, "address").lower()):
        return cam
    if create:
      cameras=self._sections()
      if "dongles" in cameras:
        cameras.remove("dongles")
      section="camera%i"%(len(cameras)+1)
      self._add_section(section)
      self._set(section, "address", address)
      return section
    return None

  #@report(level=20)
  def __cameraDict(self, items):
    # defaults
    out = {
      "reconnect_timeout": 10,
      "reconnect": False,
      "voice": False,
      "flash": False,
      "exposure": 15,
      "pan": "none"
    }
    for key, val in items:
      if key in ['reconnect_timeout', 'exposure']:
        out[key] = int(val)
      elif key in ['reconnect', 'enable', 'voice', 'flash', "enable_pincode"]:
        out[key] = val.lower() == "true"
      else:
        out[key] = val
    try:
      out["pincode"] = self.getPIN(out["address"], perfect=True, silent=False)
      out["enable_pincode"] = True
    except Exception, err:
      log.err(err)
      out["enable_pincode"] = False
    return out

  #@report(debug=True)
  def getCameras(self):
    self.reload()
    for camera in self.getCameraSections():
      yield self.__cameraDict(self._items(camera))

  def deleteCamera(self, address):
      self.reload()
      cam = self.getCameraSection(address)
      if not cam:
          return
      self._remove_section(cam)

  #@report(debug=True)
  def getCamera(self, address):
    self.reload()
    cam = self.getCameraSection(address)
    if cam:
      return self.__cameraDict(self._items(cam))
    return None

  #@report(debug=True)
  def save(self):
    self._write(open(self.name, "wb"))

  #@report(level=30)
  def __sanitizeSetting(self, key, val):
    if key in ['reconnect_timeout', 'exposure']:
      val = int(val)
    elif key in ['reconnect', 'enable', 'voice', 'flash', "enable_pincode"]:
      if type(val) != bool:
        val = val.lower() in ["true", "ok"]
    return str(val)

  #@report(level=20)
  def setCameraSetting(self, address, key, value, section=None):
    if not section:
      section = self.getCameraSection(address, True)
    if not value:
      self._remove_option(section, key)
    else:
      self._set(section, key, self.__sanitizeSetting(key, value))

  #@report(level=20)
  def setCamera(self, configuration):
    if "address" not in configuration:
      raise Exception("You need to set address")

    section = self.getCameraSection(configuration["address"], True)

    for key, value in configuration.iteritems():
      self.setCameraSetting(configuration["address"], key, value, section=section)
    self.save()
    from airi.api import UpdateManager
    UpdateManager.propagate(configuration["address"], configuration)

  # @report(debug=True)
  def setDongle(self, block, enable=False):
    self._set("dongles", block.replace(":", "_"), str(enable))

  # @report(debug=True)
  def setPIN(self, npin, block="default"):
    self._set("pin", block.replace(":", "_"), str(npin))
    from airi.api import UpdateManager
    UpdateManager.propagate(block.replace(":", "_"), {"pin": npin})

  # @report(debug=True)
  def delPIN(self, block):
    self._remove_option("pin", block.replace(":", "_"))
    from airi.api import UpdateManager
    UpdateManager.propagate(block.replace(":", "_"), {"pin": None})

  # @report(debug=True)
  def getPINs(self):
    self.reload()
    if not self.config.has_section("pin"):
      pins = {"default": default}
    else:
      pins = dict((b[0].replace("_", ":"), b[1]) for b in self.config.items("pin"))
    return pins

  # @report(debug=True)
  def getPIN(self, address=None, perfect=False, default="1234", silent=True):
    self.reload()
    if not self.config.has_section("pin"):
      pins = {"default": default}
    else:
      pins = dict((b[0].lower(), b[1]) for b in self.config.items("pin"))
    if not address:
      log.msg("No address falling back to default", level=logging.DEBUG)
      return pins.get("default", default)
    address=address.replace(":", "_").lower()

    for block in pins:
      if not perfect:
        if address.startswith(block):
          return pins[block]
      else:
        if address == block:
          return pins[block]
    if not silent:
      raise Exception("PIN not provided")
    return pins.get("default", default)

  def __getattr__(self, name):
    return getattr(self.config, name[1:])

def getSettings(name=None):
  if Settings.instance:
    return Settings.instance
  Settings.instance = Settings(name)
  return Settings.instance

if __name__=='__main__':
  import logging
  logging.basicConfig(level=logging.DEBUG)
  logging.info("starting demo")
  s=Settings("test.ini")
  logging.info(s.getDongle("00:25:BF:11:22:33"))
  s.setDongle("00:25:BF", "false")
  logging.info(s.getDongle("00:25:BF:11:22:33"))
  logging.info(s.getCamera("00:25:BF:11:22:33"))
  logging.info(s.getCamera("00:25:BF:00:25:BF"))
  s.setCamera({"address": "00:25:BF:00:25:AA", "reconnect": True})
  s.save()




# -*- coding: utf-8 -*-
from twisted.python import log
from twisted.internet import reactor
from airi.camera import dbg, CameraProtocol
from airi.settings import getSettings
from datetime import datetime
import airi.twisted_bluetooth as bluetooth

settings = getSettings()

SIZES = {
	#day modes
	"QVGA": 1,
	"VGA": 2,
	"XGA": 3,
	"QXGA": 5,
	"720P": 6,

	"QVGA Zoom 1": 11,
	"QVGA Zoom 2": 12,
	"QVGA Zoom 3": 13,
}

PANS = ["none", "right", "left", "up", "down"]

CAPABILITIES = {
	'size': [a[0] for a in sorted(SIZES.iteritems(), 
			key=lambda(x, y):(y, x))],
	'flash': True,
	'pan': PANS,
	'voice': False, # by now ;)
	'exposure': True,
	'transport': ['RFCOMM', 'L2CAP'],
	'battery': True,
}

COMMANDS = dict()


class Command():
	def __init__(self, name, timeout, command):
		self.name = name
		self.timeout = timeout
		self.command = command
		COMMANDS[name] = self

Command("date", 6, "D")
Command("flash", 1, "F")
Command("size", 3, "S")
Command("pan", 1, "P")
Command("exposure", 1, "E")


class AIRi(CameraProtocol):
	client = None
	callLater = None
	size = "QVGA"
	defaults = {
		"size": "QVGA",
		"flash": 0,
		"voice": False,
		"exposure": 0,
		"pan": "none",
	}

	def getType(self):
		return 'AIRi'

	def __init__(self, client):
		self.client = client
		self.address = client.address
		self.transport = client.transport
		self.pending = []
		self.callLater = None
		self.sco_socket = None
		self.doSetup()
		self.buffer = ""

	def internalDoCommand(self):
		if len(self.pending) == 0:
			return
		command, value, timeout = self.pending.pop(0)
		if callable(value):
			value = value()
		dbg("doCommand $%s%s" % (command, value))
		self.transport.write("$%s%s\n\r" % (command, value))
		self.callLater = reactor.callLater(timeout, 
			self.internalDoCommand)

	def doCommand(self, command, value):
		if type(command) != Command:
			command = COMMANDS[command]
		self.pending.append((command.command, value, command.timeout))
		if self.callLater and self.callLater.active():
			return
		self.internalDoCommand()

	def doSetup(self):
		dbg("doSetup")
		nsets = settings.getCamera(self.address)
		sets = self.defaults
		size = self.size
		if nsets:
			sets.update(nsets)
		dbg(sets)
		self.callLater = reactor.callLater(4, self.internalDoCommand)
		self.doCommand("date",
						lambda: datetime.utcnow().strftime("%Y%m%dT%H%M%SZ"))
		self.doCommand("size", SIZES[sets["size"]])
		self.doCommand("flash", "0" if not sets["flash"] else "1")
		self.doCommand("pan", sets["pan"].upper()[0])

	def updateSettings(self):
		self.doSetup()

	def previewData(self):
		start = self.client.buffer.find("\xff\xd8")
		end = self.client.buffer.find("\xff\xd9")
		if start == -1 or end == -1:
			return

		frame = self.client.buffer[start:end + 2]
		self.client.buffer = self.client.buffer[end + 2:]
		self.client.gotFrame(frame)

	def SCO_Data(self, sock, lost=False):
		if lost:
			reactor.removeReader(self.sco_socket)
			return
		self.buffer += sock.recv(bluetooth.sco_mtu)
		if len(self.buffer) >= 400:
			from airi.stream import MultiPartStream
			MultiPartStream.sendToClients(self.buffer,
				mime="application/octet-stream")
		 	self.buffer = ""

	def enableSCO(self):
		if self.sco_socket != None:
			return

		self.sco_socket = bluetooth.SCOReader(self.address, self.SCO_Data)
		reactor.addReader(self.sco_socket)

	def disableSCO(self):
		if not self.sco_socket:
			return
		self.sco_socket.close()
		reactor.removeReader(self.sco_socket)
		self.sco_socket = None

	def doSCO(self, enable):
		if not getattr(bluetooth, "SCOReader", None):
			dbg("sco not supported in this platform")
			return
		if enable:
			return self.enableSCO()
		else:
			return self.disableSCO()

	def dataReceived(self):
		return self.previewData()

	def disconnect(self):
		dbg("AIRi.disconnect")
		self.disableSCO()

	def set(self, option, value):
		dbg("set %s-> %s" % (option, value))
		if option == "size":
			self.doCommand(option, SIZES[value])
		elif option == "flash":
			self.doCommand(option, "0" if not value else "1")
		elif option == "pan" or option == "exposure":
			self.doCommand(option, value.upper())
		elif option == "voice":
			self.doSCO(value)
		else:
			dbg("ignored")

AIRi.Capabilities = CAPABILITIES
AIRi.Sizes = SIZES
AIRi.Panning = PANS

